/**********************************************************************************************

//  File Name       : count_gen.v 
//
//  Description     : This module implements a count pattern used to drive the CorePCS 8b10b encoder

   COPYRIGHT 2013 BY MICROSEMI 
   THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
   FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
   MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
   NO BACK-UP OF THE FILE SHOULD BE MADE. 

 
************************************************************************************************/

//******************************************************************************
//**************************** Module Declaration ******************************
//******************************************************************************
module count_gen (
  reset_n,
  clk,
  error_inject,
  data_out ,
  rxslip ,
  rxdata ,
  rxval ,
  k_out
  ) ;

  input reset_n;
  input clk;
  input error_inject ;
  input [15:0] rxdata;
  input rxval;
  output [15:0] data_out;
  output [1:0] k_out;
  output rxslip;

  reg [2:0] sm ;
  reg [7:0] c;
  reg [15:0] data_out ;
  reg [1:0] k_out;
  reg rxslip ;
  reg	     error_inject_d1 ;
  reg	     error_inject_d2 ;
  reg	     error_inject_d3 ;
  reg        error_flag;

  reg [2:0] delay_cnt;
	

always @(posedge clk or negedge reset_n)
begin
	if (~reset_n)
	begin
		error_inject_d1 <= 'b0;
		error_inject_d2 <= 'b0;
		error_inject_d3 <= 'b0;
	end
	else
	begin
		error_inject_d1 <= error_inject;
		error_inject_d2 <= error_inject_d1;
		error_inject_d3 <= error_inject_d2;
	end
end



always @ (posedge clk or negedge reset_n)
begin
	if (~reset_n)
    begin
		data_out <= 16'd0;
        k_out <= 2'b00;
        sm <= 3'b000;
        error_flag<=1'b0;
        c<=8'd0;
        rxslip <= 1'b0;
        delay_cnt <= 2'b00;
    end
	else 
    begin
    
    if (~error_inject_d3 & error_inject_d2)
        error_flag <= 1'b1;

    case (sm)
    3'b000: // send K28.5s
    begin  
      if(rxval == 1'b0) begin    
        k_out <= 2'b11;
        data_out <= {8'hbc, 8'hbc};
        rxslip <= 1'b0;
        sm <= 3'b001;
      end
    end	

    3'b001: // Check RXDATA
    begin      
      if(rxval == 1'b1) begin
        if(rxdata != {8'hbc, 8'hbc}) begin
            k_out <= 2'b01;
            rxslip <= 1'b1;
            sm <= 3'b010;
            end
        else
            begin
            sm <= 3'b011;
            end
        end
    end	

    3'b010: // RXSLIP is manually pulsed if word alignment is not achieved
    begin      
      if(delay_cnt <2'b10) begin
        delay_cnt <= delay_cnt + 1'b1;
        rxslip <= 1'b1;
        sm <= 3'b010;
        end
    else
        begin
        rxslip <= 1'b0;
        sm <= 3'b011;
        end

    end	

    3'b011: // begin sending data (count pattern)
    begin
          k_out <= 2'b00;

          if (c==8'd128) 
          begin
            c <= 8'd0;
           sm <= 3'b100;
          end
          else
            c <= c + 1;

          if (error_flag)
          begin
             data_out <= {8'hf, 8'hf};
             error_flag <= 1'b0;
          end
          else
             data_out <= {c,c};
    end

    3'b100: // 
    begin
        k_out <= 2'b11;
        data_out <= {8'hbc, 8'hbc};

         if (c == 8'd5) 
          begin
            c <= 8'd0;
           sm <= 3'b011;
          end
          else
            c <= c + 1;
    end

    default:
    begin
      sm <= 3'b000;
      c<=8'd0;
    end
    endcase
   
  end
end
     
endmodule

